aboutsummaryrefslogtreecommitdiffstats
path: root/resources/libraries
diff options
context:
space:
mode:
authorPatrik Hrnciar <phrnciar@cisco.com>2016-05-18 11:53:52 +0200
committerPatrik Hrnciar <phrnciar@cisco.com>2016-06-07 16:13:57 +0200
commit559a2b904cf95e4a5a82726d8b3791e57e7a9f9c (patch)
treefebc700ccaa624158ec3bbc774c7b2ba99845a0b /resources/libraries
parent681bb937b97b0c2bf10595e2042cf5943d61896b (diff)
iACL TCP/UDP tests
- IPv4 CSIT-92 - IPv6 CSIT-107 Change-Id: Ia93e96f624ce73ef08b89d4b22334a67ebb69e13 Signed-off-by: Patrik Hrnciar <phrnciar@cisco.com>
Diffstat (limited to 'resources/libraries')
-rw-r--r--resources/libraries/python/Classify.py147
-rw-r--r--resources/libraries/python/TrafficScriptExecutor.py10
-rw-r--r--resources/libraries/robot/traffic.robot89
3 files changed, 238 insertions, 8 deletions
diff --git a/resources/libraries/python/TrafficScriptExecutor.py b/resources/libraries/python/TrafficScriptExecutor.py
index e7b851e733..f730d3a0e3 100644
--- a/resources/libraries/python/TrafficScriptExecutor.py
+++ b/resources/libraries/python/TrafficScriptExecutor.py
@@ -48,6 +48,10 @@ class TrafficScriptExecutor(object):
:type node: dict
:type script_args: str
:type timeout: int
+ :raises RuntimeError: ICMP echo Rx timeout.
+ :raises RuntimeError: DHCP REQUEST Rx timeout.
+ :raises RuntimeError: TCP/UDP Rx timeout.
+ :raises RuntimeError: Traffic script execution failed.
"""
logger.trace("{}".format(timeout))
ssh = SSH()
@@ -66,11 +70,13 @@ class TrafficScriptExecutor(object):
logger.debug("ret_code: {}".format(ret_code))
if ret_code != 0:
if "RuntimeError: ICMP echo Rx timeout" in stderr:
- raise Exception("ICMP echo Rx timeout")
+ raise RuntimeError("ICMP echo Rx timeout")
elif "RuntimeError: DHCP REQUEST Rx timeout" in stderr:
raise RuntimeError("DHCP REQUEST Rx timeout")
+ elif "RuntimeError: TCP/UDP Rx timeout" in stderr:
+ raise RuntimeError("TCP/UDP Rx timeout")
else:
- raise Exception("Traffic script execution failed")
+ raise RuntimeError("Traffic script execution failed")
@staticmethod
def traffic_script_gen_arg(rx_if, tx_if, src_mac, dst_mac, src_ip, dst_ip):
diff --git a/resources/libraries/robot/traffic.robot b/resources/libraries/robot/traffic.robot
index c90067ad06..7dba5efbe3 100644
--- a/resources/libraries/robot/traffic.robot
+++ b/resources/libraries/robot/traffic.robot
@@ -150,3 +150,92 @@
| | ... | --tx_if | ${tx_port} | --rx_if | ${rx_port}
| | ... | --rx_arp_src_ip ${rx_arp_src_ip} | --rx_arp_dst_ip ${rx_arp_dst_ip}
| | Run Traffic Script On Node | send_icmp_check_arp.py | ${tg_node} | ${args}
+
+| Send TCP or UDP packet
+| | [Documentation] | Sends TCP or UDP packet with specified source
+| | ... | and destination port.
+| | ...
+| | ... | *Arguments:*
+| | ...
+| | ... | _NOTE:_ Arguments are based on topology:
+| | ... | TG(if1)->(if1)DUT(if2)->TG(if2)
+| | ...
+| | ... | - tg_node - Node to execute scripts on (TG). Type: dictionary
+| | ... | - src_ip - IP of source interface (TG-if1). Type: integer
+| | ... | - dst_ip - IP of destination interface (TG-if2). Type: integer
+| | ... | - tx_port - Source interface (TG-if1). Type: string
+| | ... | - tx_mac - MAC address of source interface (TG-if1). Type: string
+| | ... | - rx_port - Destionation interface (TG-if1). Type: string
+| | ... | - rx_mac - MAC address of destination interface (TG-if1). Type: string
+| | ... | - protocol - Type of protocol. Type: string
+| | ... | - source_port - Source TCP/UDP port. Type: string or integer
+| | ... | - destination_port - Destination TCP/UDP port. Type: string or integer
+| | ...
+| | ... | *Return:*
+| | ... | - No value returned
+| | ...
+| | ... | *Example:*
+| | ...
+| | ... | \| Send TCP or UDP packet \| ${nodes['TG']} \
+| | ... | \| 16.0.0.1 \| 32.0.0.1 \| eth2 \| 08:00:27:cc:4f:54 \
+| | ... | \| eth4 \| 08:00:27:c9:6a:d5 \| TCP \| 20 \| 80 \|
+| | ...
+| | [Arguments] | ${tg_node} | ${src_ip} | ${dst_ip} | ${tx_port} |
+| | ... | ${tx_mac} | ${rx_port} | ${rx_mac} | ${protocol} | ${source_port}
+| | ... | ${destination_port}
+| | ${args}= | Catenate | --tx_mac | ${tx_mac}
+| | ... | --rx_mac | ${rx_mac}
+| | ... | --src_ip | ${src_ip}
+| | ... | --dst_ip | ${dst_ip}
+| | ... | --tx_if | ${tx_port}
+| | ... | --rx_if | ${rx_port}
+| | ... | --protocol | ${protocol}
+| | ... | --source_port | ${source_port}
+| | ... | --destination_port | ${destination_port}
+| | Run Traffic Script On Node | send_tcp_udp.py
+| | ... | ${tg_node} | ${args}
+
+| Send TCP or UDP packet should failed
+| | [Documentation] | Sends TCP or UDP packet with specified source
+| | ... | and destination port.
+| | ...
+| | ... | *Arguments:*
+| | ...
+| | ... | _NOTE:_ Arguments are based on topology:
+| | ... | TG(if1)->(if1)DUT(if2)->TG(if2)
+| | ...
+| | ... | - tg_node - Node to execute scripts on (TG). Type: dictionary
+| | ... | - src_ip - IP of source interface (TG-if1). Type: integer
+| | ... | - dst_ip - IP of destination interface (TG-if2). Type: integer
+| | ... | - tx_port - Source interface (TG-if1). Type: string
+| | ... | - tx_mac - MAC address of source interface (TG-if1). Type: string
+| | ... | - rx_port - Destionation interface (TG-if1). Type: string
+| | ... | - rx_mac - MAC address of destination interface (TG-if1). Type: string
+| | ... | - protocol - Type of protocol. Type: string
+| | ... | - source_port - Source TCP/UDP port. Type: string or integer
+| | ... | - destination_port - Destination TCP/UDP port. Type: string or integer
+| | ...
+| | ... | *Return:*
+| | ... | - No value returned
+| | ...
+| | ... | *Example:*
+| | ...
+| | ... | \| Send TCP or UDP packet should failed \| ${nodes['TG']} \
+| | ... | \| 16.0.0.1 \| 32.0.0.1 \| eth2 \| 08:00:27:cc:4f:54 \
+| | ... | \| eth4 \| 08:00:27:c9:6a:d5 \| TCP \| 20 \| 80 \|
+| | ...
+| | [Arguments] | ${tg_node} | ${src_ip} | ${dst_ip} | ${tx_port} |
+| | ... | ${tx_mac} | ${rx_port} | ${rx_mac} | ${protocol} | ${source_port}
+| | ... | ${destination_port}
+| | ${args}= | Catenate | --tx_mac | ${tx_mac}
+| | ... | --rx_mac | ${rx_mac}
+| | ... | --src_ip | ${src_ip}
+| | ... | --dst_ip | ${dst_ip}
+| | ... | --tx_if | ${tx_port}
+| | ... | --rx_if | ${rx_port}
+| | ... | --protocol | ${protocol}
+| | ... | --source_port | ${source_port}
+| | ... | --destination_port | ${destination_port}
+| | Run Keyword And Expect Error | TCP/UDP Rx timeout
+| | ... | Run Traffic Script On Node | send_tcp_udp.py
+| | ... | ${tg_node} | ${args}
diff --git a/resources/libraries/python/Classify.py b/resources/libraries/python/Classify.py
index cf93a04b2b..7d62e26a67 100644
--- a/resources/libraries/python/Classify.py
+++ b/resources/libraries/python/Classify.py
@@ -30,12 +30,12 @@ class Classify(object):
:type node: dict
:type ip_version: str
:type direction: str
- :return table_index: Classify table index.
- :return skip_n: Number of skip vectors.
- :return match_n: Number of match vectors.
- :rtype table_index: int
- :rtype skip_n: int
- :rtype match_n: int
+ :return (table_index, skip_n, match_n)
+ table_index: Classify table index.
+ skip_n: Number of skip vectors.
+ match_n: Number of match vectors.
+ :rtype: tuple(int, int, int)
+ :raises RuntimeError: If VPP can't create table.
"""
output = VatExecutor.cmd_from_template(node, "classify_add_table.vat",
ip_version=ip_version,
@@ -54,6 +54,37 @@ class Classify(object):
return table_index, skip_n, match_n
@staticmethod
+ def vpp_create_classify_table_hex(node, hex_mask):
+ """Create classify table with hex mask.
+
+ :param node: VPP node to create classify table based on hex mask.
+ :param hex_mask: Classify hex mask.
+ :type node: dict
+ :type hex_mask: str
+ :return (table_index, skip_n, match_n)
+ table_index: Classify table index.
+ skip_n: Number of skip vectors.
+ match_n: Number of match vectors.
+ :rtype: tuple(int, int, int)
+ :raises RuntimeError: If VPP can't create table.
+ """
+ output = VatExecutor.cmd_from_template(node,
+ "classify_add_table_hex.vat",
+ hex_mask=hex_mask)
+
+ if output[0]["retval"] == 0:
+ table_index = output[0]["new_table_index"]
+ skip_n = output[0]["skip_n_vectors"]
+ match_n = output[0]["match_n_vectors"]
+ logger.trace('Classify table with table_index {} created on node {}'
+ .format(table_index, node['host']))
+ else:
+ raise RuntimeError('Unable to create classify table on node {}'
+ .format(node['host']))
+
+ return table_index, skip_n, match_n
+
+ @staticmethod
def vpp_configure_classify_session(node, acl_method, table_index, skip_n,
match_n, ip_version, direction, address):
"""Configuration of classify session.
@@ -84,3 +115,107 @@ class Classify(object):
ip_version=ip_version,
direction=direction,
address=address)
+
+ @staticmethod
+ def vpp_configure_classify_session_hex(node, acl_method, table_index,
+ skip_n, match_n, hex_value):
+ """Configuration of classify session with hex value.
+
+ :param node: VPP node to setup classify session.
+ :param acl_method: ACL method - deny/permit.
+ :param table_index: Classify table index.
+ :param skip_n: Number of skip vectors based on mask.
+ :param match_n: Number of match vectors based on mask.
+ :param hex_value: Classify hex value.
+ :type node: dict
+ :type acl_method: str
+ :type table_index: int
+ :type skip_n: int
+ :type match_n: int
+ :type hex_value: str
+ """
+ with VatTerminal(node) as vat:
+ vat.vat_terminal_exec_cmd_from_template(
+ "classify_add_session_hex.vat",
+ acl_method=acl_method,
+ table_index=table_index,
+ skip_n=skip_n,
+ match_n=match_n,
+ hex_value=hex_value)
+
+ @staticmethod
+ def compute_classify_hex_mask(ip_version, protocol, direction):
+ """Compute classify hex mask for TCP or UDP packet matching.
+
+ :param ip_version: Version of IP protocol.
+ :param protocol: Type of protocol.
+ :param direction: Traffic direction.
+ :type ip_version: str
+ :type protocol: str
+ :type direction: str
+ :return: Classify hex mask.
+ :rtype : str
+ :raises ValueError: If protocol is not TCP or UDP.
+ :raises ValueError: If direction is not source or destination or
+ source + destination.
+ """
+ if protocol == 'TCP' or protocol == 'UDP':
+ base_mask = Classify._compute_base_mask(ip_version)
+
+ if direction == 'source':
+ return base_mask + 'FFFF0000'
+ elif direction == 'destination':
+ return base_mask + '0000FFFF'
+ elif direction == 'source + destination':
+ return base_mask + 'FFFFFFFF'
+ else:
+ raise ValueError("Invalid direction!")
+ else:
+ raise ValueError("Invalid protocol!")
+
+ @staticmethod
+ def compute_classify_hex_value(hex_mask, source_port, destination_port):
+ """Compute classify hex value for TCP or UDP packet matching.
+
+ :param hex_mask: Classify hex mask.
+ :param source_port: Source TCP/UDP port.
+ :param destination_port: Destination TCP/UDP port.
+ :type hex_mask: str
+ :type source_port: str
+ :type destination_port: str
+ :return: Classify hex value.
+ :rtype: str
+ """
+ source_port_hex = Classify._port_convert(source_port)
+ destination_port_hex = Classify._port_convert(destination_port)
+
+ return hex_mask[:-8] + source_port_hex + destination_port_hex
+
+ @staticmethod
+ def _port_convert(port):
+ """Convert port number for classify hex table format.
+
+ :param port: TCP/UDP port number.
+ :type port: str
+ :return: TCP/UDP port number in 4-digit hexadecimal format.
+ :rtype: str
+ """
+ return '{0:04x}'.format(int(port))
+
+ @staticmethod
+ def _compute_base_mask(ip_version):
+ """Compute base classify hex mask based on IP version.
+
+ :param ip_version: Version of IP protocol.
+ :type ip_version: str
+ :return: Base hex mask.
+ :rtype: str
+ """
+ if ip_version == 'ip4':
+ return 68 * '0'
+ # base value of classify hex table for IPv4 TCP/UDP ports
+ elif ip_version == 'ip6':
+ return 108 * '0'
+ # base value of classify hex table for IPv6 TCP/UDP ports
+ else:
+ raise ValueError("Invalid IP version!")